/*
 * Mesh.cpp : Maillage 
 * Copyright (C) 2007 Florence Zara, LIRIS
 *               florence.zara@liris.univ-lyon1.fr
 *               http://liris.cnrs.fr/florence.zara/
 *
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation; either version 2.1 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/** \file Mesh.cpp
*  \brief Structures de donnes relatives au maillage 
*/



/** Librairies de base **/
#include <stdio.h>
#include <fstream>
#include <math.h> 

/** Fichiers de l application **/
#include "Noeuds.h"
#include "Mesh.h"
#include "Donnees.h"







/**
* Calcul de la longueur au repos d un ressort.
 */
void Ressort::SetLrepos() 
{
	// Variables intermediaires
	float var0, var1, var2, var3;
	
	// Calcul de la longueur du ressort entre les deux particules
	Coord a = GetParticuleA()->GetPosition();
	Coord b = GetParticuleB()->GetPosition();
	
	var1 = a.getX() - b.getX();
	var2 = a.getY() - b.getY();
	var3 = a.getZ() - b.getZ();
	
	var0 = (var1 * var1) + (var2 * var2) + (var3 * var3);
	
	// l(A,B) = sqrt( (xA-xB)≤ + (yA-yB)≤ + (zA-zB)≤ ) 	
	_Spring->_L0 = sqrt(var0);
	
}


/**
* Modification du maillage : creation d une face.
 */
void Mesh::MakeFace(Particule *p1, Particule *p2, Particule *p3, Spring *R)
{
	/* Une face est constituee de trois aretes */
	// Construction de l arete entre p1 et p2
	MakeEdge(p1, p2, R);
	
	// Construction de l arete entre p2 et p3
	MakeEdge(p2, p3, R);
	
	// Construction de l arete entre p3 et p1
	MakeEdge(p3, p1, R);
}


/**
* Modification du maillage : creation d une arete.
 */
void Mesh::MakeEdge(Particule *p1, Particule *p2, Spring *R)
{
	// Verification que le ressort entre ces deux particules n existe pas deja
	bool exist = false;
	
	// Liste des ressorts relies a p1
	std::vector<Ressort *> ressortList = p1->GetRessortList();
	
	// Parcours de la liste des ressorts de p1
	for(int i=0; i<ressortList.size(); ++i)
    {
		// Regarde si il existe un ressort relie a p1 et p2
		if ( ((ressortList[i])->GetParticuleA() == p2) || ((ressortList[i])->GetParticuleB() == p2))
		{
			// Le ressort existe deja entre p1 et p2
			exist = true;
			
			// Arret du parcours 
			break;
		}
		
    }
	
	// Ajout d un nouveau ressort dans le maillage
	if (!exist)
    {
		// Construction d un ressort entre les particules p1 et p2
		Ressort *res = new Ressort(p1, p2, R);
		
		// Ajout du ressort res dans la liste des ressorts de p1
		p1->AddRessort(res);
		
		// Ajout du ressort res dans la liste des ressorts de p2
		p2->AddRessort(res);
		
		// Ajout du ressort res dans la liste des ressorts du maillage
		this->AddRessort(res);
    }
	
}


/**
* Affichage des informations contenues dans le maillage.
 */
void Mesh::AfficheMesh()
{
	std::cout << " ____________________________________" << std::endl << std::endl;
	
	Coord tmp;
	
	// Affichage des particules
	for(int i=0; i<_ParticuleList.size(); ++i)
    {
		std::cout << "Particule numero " << GetParticule(i)->GetId() << std::endl;
		
		// Coordonnees de la particule i 
		tmp = GetParticule(i)->GetPosition();
		std::cout << "Position de la particule " << tmp.getX() << " " << tmp.getY() << " " << tmp.getZ() << std::endl;
		
		std::cout << "Nombre de voisins de la particules " << GetParticule(i)->GetNbVoisins() << std::endl << std::endl;
		std::cout << "Voisins de la particule " << std::endl;
		
		// Recuperation de la liste des ressorts reliees a la particule
		std::vector<Ressort *> ListeRessort = GetParticule(i)->GetRessortList();
		
		// Affichage des ressorts
		for(int j=0; j<ListeRessort.size(); ++j)
		{
			// Affichage du voisin
			std::cout << "           " << "Numero de la particule voisine " ;	  
			
			// Regarde si la voisine c est la particule A ou B du ressort
			if ( (ListeRessort[j]->GetParticuleA())->GetId() == GetParticule(i)->GetId() )
				std::cout << (ListeRessort[j]->GetParticuleB())->GetId() << std::endl;
			else
				std::cout << (ListeRessort[j]->GetParticuleA())->GetId() << std::endl;
			
			// Affichage des caracteristiques du ressort
			//std::cout << "           " << "Numero du ressort " << ListeRessort[j]->GetId() << std::endl;
			std::cout << "           " << "Raideur du ressort " << ListeRessort[j]->GetRaideur() << std::endl;
			std::cout << "           " << "Amortissement du ressort " << ListeRessort[j]->GetAmortissement() << std::endl;
			std::cout << "           " << "Facteur d amortissement du ressort " << ListeRessort[j]->GetFactAmorti() << std::endl;
			std::cout << "           " << "Longueur au repos " << ListeRessort[j]->GetLrepos() << std::endl << std::endl;
			
		}//for_j
		
		// Particule suivante du maillage
		std::cout << " ____________________________________" << std::endl << std::endl;
		
    }//for_i
}
